import sys
import logging
import os
import uuid

# import sentry_sdk
from flask import request, Flask
app = Flask(__name__)

COLORED_LOGS = os.environ.get("COLORED_LOGS", None)


class ColoredFormatter(logging.Formatter):
    def format(self, record):
        if record.levelno == logging.WARNING:
            record.levelname = "\033[93m%s\033[0m" % record.levelname
            record.name = "\033[93m%s\033[0m" % record.name
        elif record.levelno == logging.ERROR:
            record.levelname = "\033[91m%s\033[0m" % record.levelname
            record.name = "\033[91m%s\033[0m" % record.name
        elif record.levelno == logging.INFO:
            record.levelname = "\033[36m%s\033[0m" % record.levelname
            record.name = "\033[36m%s\033[0m" % record.name
        return logging.Formatter.format(self, record)


class RequestIDLogFilter(logging.Filter):
    """
    Log filter to inject the current request id of the request under `log_record.requestId`
    """

    def filter(self, log_record):
        if request:
            log_record.requestId = request.headers.get("x-request-id", "")
            log_record.sessionUUID = request.headers.get("x-session-uuid", "")
            # sentry_sdk.set_context("Logging IDs", {"Request Id": log_record.requestId, "Session UUID": log_record.sessionUUID})
        else:
            log_record.requestId = uuid.uuid4()
            log_record.sessionUUID = uuid.uuid4()
        return log_record


def init_logger(name, log_level=logging.INFO, logfile=None, propagate=False, **kwargs):
    logger = logging.getLogger(name)
    log_format = "[%(levelname)s %(name)s %(funcName)s requestId=%(requestId)s sessionUUID=%(sessionUUID)s] %(asctime)s - %(message)s"
    formatter = (
        ColoredFormatter(log_format) if COLORED_LOGS else logging.Formatter(log_format)
    )

    # Add file handler
    if logfile:
        fh = logging.FileHandler(logfile)
        fileFormatter = logging.Formatter(
            "[%(levelname)s %(name)s] %(asctime)s - %(message)s"
        )
        fh.setFormatter(fileFormatter)
        logger.addHandler(fh)

    # Add stream handler
    ch = logging.StreamHandler(sys.stdout)

    ch.setFormatter(formatter)
    ch.addFilter(RequestIDLogFilter())
    logger.addHandler(ch)

    logger.propagate = propagate
    logger.setLevel(log_level)
    return logger
